﻿/*
 * Generate_Beep.c
 *
 *  Created on: 2024/12/22
 *      Author: egaono-electronics
 */

#include "Generate_Beep.h"

#include def_H_Periph_PWM

#include "Time_Interval.h"



// TIM Time Base Handle構造体の宣言
TIM_HandleTypeDef    GLB_struct_TIM_Handle_Beep_PWM;



// Beep PWM初期化済みフラグ
int16_t GLB_int16_Flag_Initialize_Beep_PWM = 0;



//--------------------------------------------------
// Beepに使用するPWMの初期化
//--------------------------------------------------
// 引数:
//    なし


// 戻り値:
//    なし
//---------------------------------------------------
void Initialize_Beep_PWM(void)
{
  uint32_t uint32_PrescalerValue;


  //------------------------------------
  // 周波数 : 1kHz
  //------------------------------------
  //------------------------------------------------------------
  //----------------------------------------------------
  // Beep_PWM(TIM1)
  //----------------------------------------------------
  //------------------------------------------------------------
  uint32_PrescalerValue
    = (uint32_t)((SystemCoreClock / def_Div_Beep_PWM)
                                                / 1000000)
    - 1;
  //------------------------------------------------------------


  //------------------------------------
  // 周波数 : 1kHz
  //------------------------------------
  //------------------------------------------------------------
  // PWMの周期設定
  //------------------------------------------------------------
  // 引数;
  // TIM_HandleTypeDef *pHandle_TIMx :
  //                        Time Base Handle構造体のポインタ
  // TIM_TypeDef *TIMx : TIM選択
  // uint32_t uint32_PrescalerValue : TIMx のクロック分周比
  // uint32_t uint32_TIMx_Period : TiMx CHx の周期
  //------------------------------------------------------------
  Config_PWM_Period(&GLB_struct_TIM_Handle_Beep_PWM,
                    def_TIMx_Beep_PWM,
                    uint32_PrescalerValue,
                    (uint32_t)1000);


  //----------------------------------------------------
  // Beep_PWM : OFF
  //----------------------------------------------------
  //------------------------------------------------------------
  // PWMパルス出力設定
  //------------------------------------------------------------
  // 引数;
  // TIM_HandleTypeDef *pHandle_TIMx :
  //                        Time Base Handle構造体のポインタ
  // TIM_TypeDef *TIMx : TIM選択
  // uint32_t uint32_TIM_CHANNEL_x : Channel選択
  // uint32_t uint32_Pulse : Pulse出力値
  //------------------------------------------------------------
  Configure_PWM(&GLB_struct_TIM_Handle_Beep_PWM,
                def_TIMx_Beep_PWM,
                def_TIM_CHANNEL_x_Beep_PWM,
                (uint32_t)0);
  //----------------------------------------------------
}



//----------------------------------------------------------
// Beepを発生する。
//----------------------------------------------------------
// 引数;
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// int16_t int16_Time_Beep : Beepを発生する時間


// 戻り値;
//   -1 : 処理中
//    0 : OK終了
//    1 : NG終了
//----------------------------------------------------------
int16_t Generate_Beep(uint8_t *puint8_JobNum,
                      int16_t int16_Time_Beep)
{
  //----------------------------------------------------
  // 呼び出す関数の引数に使用する処理番号は、
  // 必ず static の変数宣言をしてください。
  //----------------------------------------------------
  static uint8_t STC_uint8_JobNum_Time;  // Time_Interval処理番号

  //------------------------------------------------------
  // Time_Interval用パラメータの構造体の宣言は、
  // 使用する関数内で、必ず static で宣言してください。
  //------------------------------------------------------
  // Time_Interval用パラメータの構造体の宣言
  //------------------------------------------------------
  static struct sParam_Time_Interval STC_structParam_Time_Interval;


  int16_t int16_Return;


  switch(*puint8_JobNum)
  {
  case 0:
    // Beep PWM初期化フラグチェック
    if(GLB_int16_Flag_Initialize_Beep_PWM <= 0){
      //------------------------------------------------
      // Beepに使用するPWMの初期化
      //------------------------------------------------
      Initialize_Beep_PWM();

      GLB_int16_Flag_Initialize_Beep_PWM = 1;
    }


    //-----------------------------------------------------------
    // Beep_PWM : 比率を50%に設定 : Beep ON
    //----------------------------------------------------------
    // CH1出力 : Beep_PWM
    Configure_PWM(&GLB_struct_TIM_Handle_Beep_PWM,
                  (TIM_TypeDef *)def_TIMx_Beep_PWM,
                  def_TIM_CHANNEL_x_Beep_PWM,
                  (uint32_t)500);
    //----------------------------------------------------------


    STC_uint8_JobNum_Time = 0;  // Time_Interval処理番号初期化

    // Beepを発生する時間をセット
    STC_structParam_Time_Interval.uint32_Interval
                                = (uint32_t)int16_Time_Beep;


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 1:
    //----------------------------------------------------------
    // 時間待ち : 単位 mSec
    //----------------------------------------------------------
    // 戻り値が -1 以外になるまで、繰り返し呼び出してください。
    //----------------------------------------------------------
    // 引数:
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // struct sParam_Time_Interval *pstructParam_Time_Interval :
    //              Time_Interval用パラメータの構造体のポインタ


    // 戻り値:
    //   -1 : 処理中 : 時間が経過していない。
    //    0 : OK終了 : 時間が経過した。
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Time_Interval(&STC_uint8_JobNum_Time,
                                 &STC_structParam_Time_Interval);
    if(int16_Return == -1){
      // 処理中 : 時間が経過していない。
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    //----------------------------------------------------------
    // Beep_PWM : OFF
    //----------------------------------------------------------
    // CH1出力 : Beep_PWM
    Configure_PWM(&GLB_struct_TIM_Handle_Beep_PWM,
                  (TIM_TypeDef *)def_TIMx_Beep_PWM,
                  def_TIM_CHANNEL_x_Beep_PWM,
                  (uint32_t)0);
    //----------------------------------------------------------


    *puint8_JobNum = 0;  // 処理番号初期化 : case 0: に移行する。
    int16_Return = 0;    // OK終了
    break;

  default:
    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 1;    // NG
    break;
  }

  return(int16_Return);
}



//----------------------------------------------------------
// Beepを指定回数、発生する。
//----------------------------------------------------------
// 引数;
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// int16_t int16_Times : Beepを発生する回数
// int16_t int16_Time_Beep : Beepを発生する時間


// 戻り値;
//   -1 : 処理中
//    0 : OK終了
//    1 : NG終了
//----------------------------------------------------------
int16_t Beep_Repeat(uint8_t *puint8_JobNum,
                    int16_t int16_Times,
                    int16_t int16_Time_Beep)
{
  //----------------------------------------------------
  // 呼び出す関数の引数に使用する処理番号は、
  // 必ず static の変数宣言をしてください。
  //----------------------------------------------------
  static uint8_t STC_uint8_JobNum_Time;  // Time_Interval処理番号

  //------------------------------------------------------
  // Time_Interval用パラメータの構造体の宣言は、
  // 使用する関数内で、必ず static で宣言してください。
  //------------------------------------------------------
  // Time_Interval用パラメータの構造体の宣言
  //------------------------------------------------------
  static struct sParam_Time_Interval STC_structParam_Time_Interval;

  static int16_t STC_int16_Repeat;  // 繰り返し回数


  int16_t int16_Return;


  switch(*puint8_JobNum)
  {
  case 0:
    // Beep PWM初期化フラグチェック
    if(GLB_int16_Flag_Initialize_Beep_PWM <= 0){
      //------------------------------------------------
      // Beepに使用するPWMの初期化
      //------------------------------------------------
      Initialize_Beep_PWM();

      GLB_int16_Flag_Initialize_Beep_PWM = 1;
    }


    // Beep繰り返し回数をセット
    STC_int16_Repeat = int16_Times;


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 1:
    //--------------------------------------
    // Beep ON処理
    //--------------------------------------
    //----------------------------------------------------------
    // Beep_PWM : 比率を50%に設定 : Beep ON
    //----------------------------------------------------------
    // CH1出力 : Beep_PWM
    Configure_PWM(&GLB_struct_TIM_Handle_Beep_PWM,
                  (TIM_TypeDef *)def_TIMx_Beep_PWM,
                  def_TIM_CHANNEL_x_Beep_PWM,
                  (uint32_t)500);
    //----------------------------------------------------------


    STC_uint8_JobNum_Time = 0;  // Time_Interval処理番号初期化

    // Beepを発生する時間をセット
    STC_structParam_Time_Interval.uint32_Interval
                                = (uint32_t)int16_Time_Beep;


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 2:
    //----------------------------------------------------------
    // 時間待ち : 単位 mSec
    //----------------------------------------------------------
    // 戻り値が -1 以外になるまで、繰り返し呼び出してください。
    //----------------------------------------------------------
    // 引数:
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // struct sParam_Time_Interval *pstructParam_Time_Interval :
    //              Time_Interval用パラメータの構造体のポインタ


    // 戻り値:
    //   -1 : 処理中 : 時間が経過していない。
    //    0 : OK終了 : 時間が経過した。
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Time_Interval(&STC_uint8_JobNum_Time,
                                 &STC_structParam_Time_Interval);
    if(int16_Return == -1){
      // 処理中 : 時間が経過していない。
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    //----------------------------------------------------------
    // Beep_PWM : OFF
    //----------------------------------------------------------
    // CH1出力 : Beep_PWM
    Configure_PWM(&GLB_struct_TIM_Handle_Beep_PWM,
                  (TIM_TypeDef *)def_TIMx_Beep_PWM,
                  def_TIM_CHANNEL_x_Beep_PWM,
                  (uint32_t)0);
    //----------------------------------------------------------


    STC_int16_Repeat--;  // 繰り返し回数をデクリメントする。

    // 終了チェック
    if(STC_int16_Repeat <= 0){
      // 終了
      *puint8_JobNum = 0;  // 処理番号初期化
      int16_Return = 0;    // OK終了
      break;
    }


    STC_uint8_JobNum_Time = 0;  // Time_Interval処理番号初期化

    // Beepを発生する時間をセット
    STC_structParam_Time_Interval.uint32_Interval
                                = (uint32_t)int16_Time_Beep;


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 3:
    //----------------------------------------------------------
    // 時間待ち : 単位 mSec
    //----------------------------------------------------------
    // 戻り値が -1 以外になるまで、繰り返し呼び出してください。
    //----------------------------------------------------------
    // 引数:
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // struct sParam_Time_Interval *pstructParam_Time_Interval :
    //              Time_Interval用パラメータの構造体のポインタ


    // 戻り値:
    //   -1 : 処理中 : 時間が経過していない。
    //    0 : OK終了 : 時間が経過した。
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Time_Interval(&STC_uint8_JobNum_Time,
                                 &STC_structParam_Time_Interval);
    if(int16_Return == -1){
      // 処理中 : 時間が経過していない。
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    *puint8_JobNum = 1;  // Beep ON処理に移行する。: case 1:
                         //             : Beep ON を繰り返す。
    int16_Return = -1;   // 処理継続
    break;

  default:
    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 1;    // NG
    break;
  }

  return(int16_Return);
}
